from typing import Union, Mapping

from setuptools import build_meta
from setuptools.build_meta import *

import requirements


def get_requires_for_build_wheel(
    config_settings: Union[Mapping[str, Union[str, list[str], None]], None] = None,
) -> list[str]:
    return [
        *build_meta.get_requires_for_build_wheel(config_settings),
        "wheel",
        *requirements.get_build_dependencies(),
    ]


def get_requires_for_build_editable(
    config_settings: Union[Mapping[str, Union[str, list[str], None]], None] = None,
) -> list[str]:
    return [
        *build_meta.get_requires_for_build_editable(config_settings),
        *requirements.get_build_dependencies(),
    ]



import os
import shutil
import zipfile
import json
from urllib.request import urlopen, Request
import io
from typing import Generator, TypeVar, Any, Optional
import logging

from amulet.utils.task_manager import AbstractProgressManager, VoidProgressManager
from amulet.resource_pack import JavaResourcePack

T = TypeVar("T")

log = logging.getLogger(__name__)

launcher_manifest: Optional[dict] = None
INCLUDE_SNAPSHOT = False


def get_launcher_manifest() -> dict:
    global launcher_manifest
    if launcher_manifest is None:
        log.info("Downloading java launcher manifest file.")
        with urlopen(
            "https://launchermeta.mojang.com/mc/game/version_manifest.json", timeout=20
        ) as manifest:
            launcher_manifest = json.load(manifest)
        log.info("Finished downloading java launcher manifest file.")
    return launcher_manifest


def get_latest(
    progress_manager: AbstractProgressManager = VoidProgressManager(),
) -> JavaResourcePack:
    """Download the latest resource pack if required.

    :return: The loaded Java resource pack.
    """
    vanilla_rp_path = os.path.join(
        os.environ["CACHE_DIR"], "resource_packs", "java", "vanilla"
    )
    try:
        if INCLUDE_SNAPSHOT:
            new_version = get_launcher_manifest()["latest"]["snapshot"]
        else:
            new_version = get_launcher_manifest()["latest"]["release"]
    except Exception as e:
        if os.path.isdir(vanilla_rp_path):
            log.error(
                "Could not download the launcher manifest. The resource pack seems to be present so using that."
            )
        else:
            raise e
    else:
        has_new_pack = False
        if os.path.isfile(os.path.join(vanilla_rp_path, "version")):
            with open(os.path.join(vanilla_rp_path, "version")) as f:
                old_version = f.read()
            has_new_pack = old_version == new_version

        if not has_new_pack:
            _remove_and_download(vanilla_rp_path, new_version, progress_manager)
    return JavaResourcePack(vanilla_rp_path)


_java_vanilla_fix: Optional[JavaResourcePack] = None
_java_vanilla_latest: Optional[JavaResourcePack] = None


def get_java_vanilla_fix() -> JavaResourcePack:
    global _java_vanilla_fix
    if _java_vanilla_fix is None:
        _java_vanilla_fix = JavaResourcePack(
            os.path.join(os.path.dirname(__file__), "java_vanilla_fix")
        )
    return _java_vanilla_fix


def get_java_vanilla_latest(
    progress_manager: AbstractProgressManager = VoidProgressManager(),
) -> JavaResourcePack:
    global _java_vanilla_latest
    if _java_vanilla_latest is None:
        _java_vanilla_latest = get_latest(progress_manager)
    return _java_vanilla_latest


def _remove_and_download(
    path: str,
    version: str,
    progress_manager: AbstractProgressManager = VoidProgressManager(),
) -> None:
    # try downloading the new resources to a temporary location
    temp_path = os.path.join(os.path.dirname(path), "_temp_")
    # clear the temporary location
    if os.path.isfile(temp_path):
        os.remove(temp_path)
    elif os.path.isdir(temp_path):
        shutil.rmtree(temp_path, ignore_errors=True)

    _download_resources(temp_path, version, progress_manager)
    if os.path.isdir(path):
        shutil.rmtree(path, ignore_errors=True)

    shutil.move(temp_path, path)

    with open(os.path.join(path, "version"), "w") as f:
        f.write(version)


def _download_with_retry(
    url: str,
    chunk_size: int = 4096,
    attempts: int = 5,
    progress_manager: AbstractProgressManager = VoidProgressManager(),
) -> bytes:
    content_length_found = 0
    content = []

    for _ in range(attempts):
        request = Request(url, headers={"Range": f"bytes={content_length_found}-"})
        with urlopen(request, timeout=20) as response:
            content_length = int(response.headers["content-length"].strip())
            while content_length_found < content_length:
                chunk = response.read(chunk_size)
                if not chunk:
                    break
                content.append(chunk)
                content_length_found += len(chunk)
                progress_manager.update_progress(
                    min(1.0, content_length_found / content_length)
                )
        if content_length == content_length_found:
            break
    else:
        raise RuntimeError(f"Failed to download")
    return b"".join(content)


def _download_resources(
    path: str,
    version: str,
    progress_manager: AbstractProgressManager = VoidProgressManager(),
    chunk_size: int = 4096,
) -> None:
    log.info(f"Downloading Java resource pack for version {version}")
    version_url = next(
        (v["url"] for v in get_launcher_manifest()["versions"] if v["id"] == version),
        None,
    )
    if version_url is None:
        raise Exception(f"Could not find Java resource pack for version {version}.")

    try:
        with urlopen(version_url, timeout=20) as vm:
            version_manifest = json.load(vm)
        version_client_url = version_manifest["downloads"]["client"]["url"]

        progress_manager_child_1 = progress_manager.get_child(0.0, 0.5)

        data = _download_with_retry(
            version_client_url,
            chunk_size=chunk_size,
            progress_manager=progress_manager_child_1,
        )

        client = zipfile.ZipFile(io.BytesIO(data))
        paths: list[str] = [
            fpath for fpath in client.namelist() if fpath.startswith("assets/")
        ]
        path_count = len(paths)
        progress_manager_child_2 = progress_manager.get_child(0.5, 1.0)
        for path_index, fpath in enumerate(paths):
            if not path_index % 30:
                progress_manager_child_2.update_progress(path_index / path_count)
            if fpath.endswith("/"):
                continue
            os.makedirs(
                os.path.dirname(os.path.abspath(os.path.join(path, fpath))),
                exist_ok=True,
            )
            client.extract(fpath, path)
        if "pack.mcmeta" in client.namelist():
            client.extract("pack.mcmeta", path)
        else:
            # TODO: work out proper version support for this
            with open(os.path.join(path, "pack.mcmeta"), "w") as f:
                f.write(
                    '{"pack": {"description": "The default data for Minecraft","pack_format": 7}}'
                )
        if "pack.png" in client.namelist():
            client.extract("pack.png", path)

    except Exception as e:
        log.error(
            f"Failed to download and extract the Java resource pack for version {version}.",
            exc_info=True,
        )
        raise e
    log.info(f"Finished downloading Java resource pack for version {version}")



from amulet.resource_pack.abc.resource_pack import BaseResourcePack


class UnknownResourcePack(BaseResourcePack):
    def __repr__(self) -> str:
        return f"UnknownResourcePack({self._root_dir})"

    @staticmethod
    def is_valid(pack_path: str) -> bool:
        return True



from __future__ import annotations

from amulet.resource_pack._load import load_resource_pack, load_resource_pack_manager
from amulet.resource_pack.abc.resource_pack import BaseResourcePack
from amulet.resource_pack.abc.resource_pack_manager import BaseResourcePackManager
from amulet.resource_pack.java.resource_pack import JavaResourcePack
from amulet.resource_pack.java.resource_pack_manager import JavaResourcePackManager
from amulet.resource_pack.unknown_resource_pack import UnknownResourcePack

from . import (
    _amulet_resource_pack,
    _load,
    _version,
    abc,
    image,
    java,
    mesh,
    unknown_resource_pack,
)

__all__ = [
    "BaseResourcePack",
    "BaseResourcePackManager",
    "JavaResourcePack",
    "JavaResourcePackManager",
    "UnknownResourcePack",
    "abc",
    "compiler_config",
    "image",
    "java",
    "load_resource_pack",
    "load_resource_pack_manager",
    "mesh",
    "unknown_resource_pack",
]

def _init() -> None: ...

__version__: str
compiler_config: dict



from __future__ import annotations

from amulet.resource_pack.abc.resource_pack import BaseResourcePack
from amulet.resource_pack.abc.resource_pack_manager import BaseResourcePackManager

from . import resource_pack, resource_pack_manager

__all__ = [
    "BaseResourcePack",
    "BaseResourcePackManager",
    "resource_pack",
    "resource_pack_manager",
]



import os
import json
from typing import Union, Iterable, Iterator, Optional
from PIL import Image
import numpy
import glob
import itertools
import logging
import re

from amulet.nbt import StringTag

from amulet.utils.cast import dynamic_cast
from amulet.utils.task_manager import AbstractProgressManager
from amulet.core.block import Block
from amulet.resource_pack import BaseResourcePackManager
from amulet.resource_pack.java import JavaResourcePack
from amulet.resource_pack.mesh.block import (
    BlockMesh,
    BlockMeshPart,
    Triangle,
    Vertex,
    FloatVec3,
    FloatVec2,
    BlockMeshTransparency,
    BlockMeshCullDirection,
    merge_block_meshes,
    FACE_KEYS,
    CUBE_FACE_LUT,
    UV_ROTATION_LUT,
    TRI_FACE,
)
from amulet.resource_pack.mesh.util import rotate_3d

log = logging.getLogger(__name__)


UselessImageGroups = {
    "colormap",
    "effect",
    "environment",
    "font",
    "gui",
    "map",
    "mob_effect",
    "particle",
}

_PropertiesPattern = re.compile(r"(?P<name>[a-zA-Z0-9_]+)=(?P<value>[a-zA-Z0-9_]+),?")

CULL_DIRECTIONS = {
    None: BlockMeshCullDirection.CullNone,
    "down": BlockMeshCullDirection.CullDown,
    "up": BlockMeshCullDirection.CullUp,
    "north": BlockMeshCullDirection.CullNorth,
    "east": BlockMeshCullDirection.CullEast,
    "south": BlockMeshCullDirection.CullSouth,
    "west": BlockMeshCullDirection.CullWest,
}


def get_py_data(obj: Block.PropertyValue) -> str | bytes | int:
    if isinstance(obj, StringTag):
        return obj.py_str_or_bytes
    else:
        return obj.py_int


class JavaResourcePackManager(BaseResourcePackManager[JavaResourcePack]):
    """A class to load and handle the data from the packs.
    Packs are given as a list with the later packs overwriting the earlier ones."""

    def __init__(
        self,
        resource_packs: Union[JavaResourcePack, Iterable[JavaResourcePack]],
        load: bool = True,
    ) -> None:
        super().__init__()
        self._blockstate_files: dict[tuple[str, str], dict] = {}
        self._textures: dict[tuple[str, str], str] = {}
        self._texture_is_transparent: dict[str, tuple[float, bool]] = {}
        self._model_files: dict[tuple[str, str], dict] = {}
        if isinstance(resource_packs, Iterable):
            self._packs = list(resource_packs)
        elif isinstance(resource_packs, JavaResourcePack):
            self._packs = [resource_packs]
        else:
            raise Exception(f"Invalid format {resource_packs}")
        if load:
            self.reload()

    def _unload(self) -> None:
        """Clear all loaded resources."""
        super()._unload()
        self._blockstate_files.clear()
        self._textures.clear()
        self._texture_is_transparent.clear()
        self._model_files.clear()

    def _load(self, progress_manager: AbstractProgressManager) -> None:
        blockstate_file_paths: dict[tuple[str, str], str] = {}
        model_file_paths: dict[tuple[str, str], str] = {}

        transparency_cache_path = os.path.join(
            os.environ["CACHE_DIR"], "resource_packs", "java", "transparency_cache.json"
        )
        self._load_transparency_cache(transparency_cache_path)

        self._textures[("minecraft", "missing_no")] = self.missing_no

        pack_count = len(self._packs)

        for pack_index, pack in enumerate(self._packs):
            # pack_format=2 textures/blocks, textures/items - case sensitive
            # pack_format=3 textures/blocks, textures/items - lower case
            # pack_format=4 textures/block, textures/item
            # pack_format=5 model paths and texture paths are now optionally namespaced

            pack_progress_manager = progress_manager.get_child(
                max(0.0, pack_index / pack_count),
                min(1.0, (1 + pack_index) / pack_count),
            )
            pack_progress_manager.update_progress(0.0)

            if pack.valid_pack and pack.pack_format >= 2:
                image_progress_manager = pack_progress_manager.get_child(0.0, 1 / 3)
                image_paths = glob.glob(
                    os.path.join(
                        glob.escape(pack.root_dir),
                        "assets",
                        "*",  # namespace
                        "textures",
                        "**",
                        "*.png",
                    ),
                    recursive=True,
                )
                image_count = len(image_paths)
                for image_index, texture_path in enumerate(image_paths):
                    _, namespace, _, *rel_path_list = os.path.normpath(
                        os.path.relpath(texture_path, pack.root_dir)
                    ).split(os.sep)
                    if rel_path_list[0] not in UselessImageGroups:
                        rel_path = "/".join(rel_path_list)[:-4]
                        self._textures[(namespace, rel_path)] = texture_path
                        if (
                            os.stat(texture_path).st_mtime
                            != self._texture_is_transparent.get(texture_path, [0])[0]
                        ):
                            im: Image.Image = Image.open(texture_path)
                            if im.mode == "RGBA":
                                alpha = numpy.array(im.getchannel("A").getdata())
                                texture_is_transparent = bool(numpy.any(alpha != 255))
                            elif im.mode == "P":
                                texture_is_transparent = (
                                    im.info.get("transparency", 0) != 0
                                )
                            else:
                                texture_is_transparent = False

                            self._texture_is_transparent[texture_path] = (
                                os.stat(texture_path).st_mtime,
                                texture_is_transparent,
                            )
                    if not image_index % 100:
                        image_progress_manager.update_progress(
                            image_index / image_count
                        )

                blockstate_progress_manager = pack_progress_manager.get_child(
                    1 / 3, 2 / 3
                )
                blockstate_paths = glob.glob(
                    os.path.join(
                        glob.escape(pack.root_dir),
                        "assets",
                        "*",  # namespace
                        "blockstates",
                        "*.json",
                    )
                )
                blockstate_count = len(blockstate_paths)
                for blockstate_index, blockstate_path in enumerate(blockstate_paths):
                    _, namespace, _, blockstate_file = os.path.normpath(
                        os.path.relpath(blockstate_path, pack.root_dir)
                    ).split(os.sep)
                    blockstate_file_paths[(namespace, blockstate_file[:-5])] = (
                        blockstate_path
                    )
                    if not blockstate_index % 100:
                        blockstate_progress_manager.update_progress(
                            blockstate_index / blockstate_count
                        )

                model_progress_manager = pack_progress_manager.get_child(2 / 3, 1.0)
                model_paths = glob.glob(
                    os.path.join(
                        glob.escape(pack.root_dir),
                        "assets",
                        "*",  # namespace
                        "models",
                        "**",
                        "*.json",
                    ),
                    recursive=True,
                )
                model_count = len(model_paths)
                for model_index, model_path in enumerate(model_paths):
                    _, namespace, _, *rel_path_list = os.path.normpath(
                        os.path.relpath(model_path, pack.root_dir)
                    ).split(os.sep)
                    rel_path = "/".join(rel_path_list)[:-5]
                    model_file_paths[(namespace, rel_path.replace(os.sep, "/"))] = (
                        model_path
                    )
                    if not model_index % 100:
                        model_progress_manager.update_progress(
                            model_index / model_count
                        )

        os.makedirs(os.path.dirname(transparency_cache_path), exist_ok=True)
        with open(transparency_cache_path, "w") as f:
            json.dump(self._texture_is_transparent, f)

        for key, path in blockstate_file_paths.items():
            with open(path) as fi:
                try:
                    self._blockstate_files[key] = json.load(fi)
                except json.JSONDecodeError:
                    log.error(f"Failed to parse blockstate file {path}")

        for key, path in model_file_paths.items():
            with open(path) as fi:
                try:
                    self._model_files[key] = json.load(fi)
                except json.JSONDecodeError:
                    log.error(f"Failed to parse model file file {path}")

    @property
    def textures(self) -> tuple[str, ...]:
        """Returns a tuple of all the texture paths in the resource pack."""
        return tuple(self._textures.values())

    def get_texture_path(self, namespace: Optional[str], relative_path: str) -> str:
        """Get the absolute texture path from the namespace and relative path pair"""
        if namespace is None:
            return self.missing_no
        key = (namespace, relative_path)
        if key in self._textures:
            return self._textures[key]
        else:
            return self.missing_no

    @staticmethod
    def parse_state_val(val: Union[str, bool]) -> list:
        """Convert the json block state format into a consistent format."""
        if isinstance(val, str):
            return [StringTag(v) for v in val.split("|")]
        elif isinstance(val, bool):
            return [StringTag("true") if val else StringTag("false")]
        else:
            raise Exception(f"Could not parse state val {val}")

    def _get_model(self, block: Block) -> BlockMesh:
        """Find the model paths for a given block state and load them."""
        if (block.namespace, block.base_name) in self._blockstate_files:
            blockstate: dict = self._blockstate_files[
                (block.namespace, block.base_name)
            ]
            if "variants" in blockstate:
                for variant in blockstate["variants"]:
                    if variant == "":
                        try:
                            return self._load_blockstate_model(
                                blockstate["variants"][variant]
                            )
                        except Exception as e:
                            log.exception(
                                f"Failed to load block model {blockstate['variants'][variant]}\n{e}"
                            )
                    else:
                        properties_match = _PropertiesPattern.finditer(f",{variant}")
                        if all(
                            get_py_data(
                                block.properties.get(
                                    match.group("name"),
                                    StringTag(match.group("value")),
                                )
                            )
                            == match.group("value")
                            for match in properties_match
                        ):
                            try:
                                return self._load_blockstate_model(
                                    blockstate["variants"][variant]
                                )
                            except Exception as e:
                                log.exception(
                                    f"Failed to load block model {blockstate['variants'][variant]}\n{e}"
                                )

            elif "multipart" in blockstate:
                models = []

                for case in blockstate["multipart"]:
                    try:
                        if "when" in case:
                            if "OR" in case["when"]:
                                if not any(
                                    all(
                                        block.properties.get(prop, None)
                                        in self.parse_state_val(val)
                                        for prop, val in prop_match.items()
                                    )
                                    for prop_match in case["when"]["OR"]
                                ):
                                    continue
                            elif "AND" in case["when"]:
                                if not all(
                                    all(
                                        block.properties.get(prop, None)
                                        in self.parse_state_val(val)
                                        for prop, val in prop_match.items()
                                    )
                                    for prop_match in case["when"]["AND"]
                                ):
                                    continue
                            elif not all(
                                block.properties.get(prop, None)
                                in self.parse_state_val(val)
                                for prop, val in case["when"].items()
                            ):
                                continue

                        if "apply" in case:
                            try:
                                models.append(
                                    self._load_blockstate_model(case["apply"])
                                )

                            except Exception as e:
                                log.exception(
                                    f"Failed to load block model {case['apply']}\n{e}"
                                )
                    except Exception as e:
                        log.exception(f"Failed to parse block state for {block}\n{e}")

                return merge_block_meshes(models)

        return self.missing_block

    def _load_blockstate_model(
        self, blockstate_value: Union[dict, list[dict]]
    ) -> BlockMesh:
        """Load the model(s) associated with a block state and apply rotations if needed."""
        if isinstance(blockstate_value, list):
            blockstate_value = blockstate_value[0]
        if "model" not in blockstate_value:
            return self.missing_block
        model_path = blockstate_value["model"]
        rotx = int(blockstate_value.get("x", 0) // 90)
        roty = int(blockstate_value.get("y", 0) // 90)
        uvlock = blockstate_value.get("uvlock", False)

        model = self._load_block_model(model_path)

        # TODO: rotate model based on uv_lock
        return model.rotate(rotx, roty)

    def _load_block_model(self, model_path: str) -> BlockMesh:
        """Load the model file associated with the Block and convert to a BlockMesh."""
        # recursively load model files into one dictionary
        java_model = self._recursive_load_block_model(model_path)

        if set(java_model.get("textures", {})).difference(
            {"particle"}
        ) and not java_model.get("elements"):
            return self.missing_block

        # set up some variables
        texture_paths: dict[str, int] = {}
        mesh_parts: list[tuple[list[Vertex], list[Triangle]] | None] = [
            None,
            None,
            None,
            None,
            None,
            None,
            None,
        ]
        transparency = BlockMeshTransparency.Partial

        for element in dynamic_cast(java_model.get("elements", []), list):
            # iterate through elements (one cube per element)
            element_faces = dynamic_cast(element.get("faces", {}), dict)

            opaque_face_count = 0
            if (
                transparency
                and "rotation" not in element
                and element.get("to", [16, 16, 16]) == [16, 16, 16]
                and element.get("from", [0, 0, 0]) == [0, 0, 0]
                and len(element_faces) >= 6
            ):
                # if the block is not yet defined as a solid block
                # and this element is a full size element
                # check if the texture is opaque
                transparency = BlockMeshTransparency.FullTranslucent
                check_faces = True
            else:
                check_faces = False

            # lower and upper box coordinates
            x1, y1, z1 = [v / 16.0 for v in element.get("from", [0, 0, 0])]
            x2, y2, z2 = [v / 16.0 for v in element.get("to", [16, 16, 16])]

            # vertex coordinates of the box
            box_coordinates = numpy.array(
                list(
                    itertools.product(
                        (min(x1, x2), max(x1, x2)),
                        (min(y1, y2), max(y1, y2)),
                        (min(z1, z2), max(z1, x2)),
                    )
                )
            )

            for face_dir, face_data in element_faces.items():
                if face_dir not in CUBE_FACE_LUT:
                    continue

                # get the cull direction. If there is an opaque block in this direction then cull this face
                cull_dir = face_data.get("cullface", None)
                if cull_dir not in FACE_KEYS:
                    cull_dir = None

                # get the relative texture path for the texture used
                texture_relative_path = face_data.get("texture", None)
                while isinstance(
                    texture_relative_path, str
                ) and texture_relative_path.startswith("#"):
                    texture_relative_path = java_model["textures"].get(
                        texture_relative_path[1:], None
                    )
                texture_path_list = texture_relative_path.split(":", 1)
                if len(texture_path_list) == 2:
                    namespace, texture_relative_path = texture_path_list
                else:
                    namespace = "minecraft"

                texture_path = self.get_texture_path(namespace, texture_relative_path)

                if check_faces:
                    if self._texture_is_transparent[texture_path][1]:
                        check_faces = False
                    else:
                        opaque_face_count += 1

                # texture index for the face
                texture_index = texture_paths.setdefault(
                    texture_path, len(texture_paths)
                )

                # get the uv values for each vertex
                texture_uv: tuple[float, float, float, float]
                if "uv" in face_data:
                    uv = face_data["uv"]
                    texture_uv = (
                        uv[0] / 16.0,
                        uv[1] / 16.0,
                        uv[2] / 16.0,
                        uv[3] / 16.0,
                    )
                else:
                    # TODO: get the uv based on box location if not defined
                    texture_uv = (0.0, 0.0, 1.0, 1.0)

                texture_rotation = face_data.get("rotation", 0)
                uv_slice = (
                    UV_ROTATION_LUT[2 * int(texture_rotation / 90) :]
                    + UV_ROTATION_LUT[: 2 * int(texture_rotation / 90)]
                )

                # merge the vertex coordinates and texture coordinates
                face_verts = box_coordinates[CUBE_FACE_LUT[face_dir]]
                if "rotation" in element:
                    rotation = element["rotation"]
                    origin = [r / 16 for r in rotation.get("origin", [8, 8, 8])]
                    angle = rotation.get("angle", 0)
                    axis = rotation.get("axis", "x")
                    angles = [0, 0, 0]
                    if axis == "x":
                        angles[0] = -angle
                    elif axis == "y":
                        angles[1] = -angle
                    elif axis == "z":
                        angles[2] = -angle
                    face_verts = rotate_3d(face_verts, *angles, *origin)

                cull_direction = CULL_DIRECTIONS[cull_dir]
                part = mesh_parts[cull_direction]
                if part is None:
                    mesh_parts[cull_direction] = part = ([], [])
                verts, tris = part
                vert_count = len(verts)

                if "tintindex" in face_data:
                    # TODO: set this up for each supported block
                    tint_vec = FloatVec3(0, 1, 0)
                else:
                    tint_vec = FloatVec3(1, 1, 1)

                for i in range(4):
                    x, y, z = face_verts[i]
                    uvx = texture_uv[uv_slice[i * 2]]
                    uvy = texture_uv[uv_slice[i * 2 + 1]]
                    verts.append(
                        Vertex(
                            FloatVec3(x, y, z),
                            FloatVec2(uvx, uvy),
                            tint_vec,
                        )
                    )

                for a, b, c in TRI_FACE.reshape((2, 3)):
                    tris.append(
                        Triangle(
                            a + vert_count,
                            b + vert_count,
                            c + vert_count,
                            texture_index,
                        )
                    )

            if opaque_face_count == 6:
                transparency = BlockMeshTransparency.FullOpaque

        def create_part(
            part: tuple[list[Vertex], list[Triangle]] | None,
        ) -> BlockMeshPart | None:
            return None if part is None else BlockMeshPart(*part)

        return BlockMesh(
            transparency,
            list(texture_paths),
            (
                create_part(mesh_parts[0]),
                create_part(mesh_parts[1]),
                create_part(mesh_parts[2]),
                create_part(mesh_parts[3]),
                create_part(mesh_parts[4]),
                create_part(mesh_parts[5]),
                create_part(mesh_parts[6]),
            ),
        )

    def _recursive_load_block_model(self, model_path: str) -> dict:
        """Load a model json file and recursively load and merge the parent entries into one json file."""
        model_path_list = model_path.split(":", 1)
        if len(model_path_list) == 2:
            namespace, model_path = model_path_list
        else:
            namespace = "minecraft"
        if (namespace, model_path) in self._model_files:
            model = self._model_files[(namespace, model_path)]

            if "parent" in model:
                parent_model = self._recursive_load_block_model(model["parent"])
            else:
                parent_model = {}
            if "textures" in model:
                if "textures" not in parent_model:
                    parent_model["textures"] = {}
                for key, val in model["textures"].items():
                    parent_model["textures"][key] = val
            if "elements" in model:
                parent_model["elements"] = model["elements"]

            return parent_model

        return {}



from typing import TypeAlias
import numpy
import itertools

from amulet.resource_pack.mesh.block import (
    BlockMesh,
    BlockMeshPart,
    Triangle,
    Vertex,
    FloatVec3,
    FloatVec2,
    BlockMeshTransparency,
    BlockMeshCullDirection,
)

BoundsType: TypeAlias = tuple[
    tuple[float, float], tuple[float, float], tuple[float, float]
]
TextureUVType: TypeAlias = tuple[
    tuple[float, float, float, float],
    tuple[float, float, float, float],
    tuple[float, float, float, float],
    tuple[float, float, float, float],
    tuple[float, float, float, float],
    tuple[float, float, float, float],
]

UNIT_BOX_COORDINATES = numpy.array(
    list(itertools.product((0, 1), (0, 1), (0, 1)))
)  # X, Y, Z

CULL_DIRECTION_NAMES = (
    "down",
    "up",
    "north",
    "east",
    "south",
    "west",
)

# This maps face direction to the vertices used (defined in UNIT_BOX_COORDINATES)
VERTEX_INDEXES = (
    numpy.array([0, 4, 5, 1]),
    numpy.array([3, 7, 6, 2]),
    numpy.array([4, 0, 2, 6]),
    numpy.array([5, 4, 6, 7]),
    numpy.array([1, 5, 7, 3]),
    numpy.array([0, 1, 3, 2]),
)

# This maps face direction to the vertices used (defined in UNIT_BOX_COORDINATES)
CUBE_FACE_LUT = dict(zip(CULL_DIRECTION_NAMES, VERTEX_INDEXES))

TRI_FACE = numpy.array([0, 1, 2, 0, 2, 3], numpy.uint32)

# cube_vert_lut = {  # This maps from vertex index to index in [minx, miny, minz, maxx, maxy, maxz]
# 	1: [0, 1, 5],
# 	3: [0, 4, 5],
# 	0: [0, 1, 2],
# 	2: [0, 4, 2],
# 	5: [3, 1, 5],
# 	7: [3, 4, 5],
# 	4: [3, 1, 2],
# 	6: [3, 4, 2],
# }
#
# # combines the above two to map from face to index in [minx, miny, minz, maxx, maxy, maxz]. Used to index a numpy array
# # The above two have been kept separate because the merged result is unintuitive and difficult to edit.
# cube_lut = {
# 	face_dir_: [
# 		vert_coord_ for vert_ in vert_index_ for vert_coord_ in cube_vert_lut[vert_]
# 	]
# 	for face_dir_, vert_index_ in CUBE_FACE_LUT.items()
# }

UV_ROTATION_LUT = [0, 3, 2, 3, 2, 1, 0, 1]  # remap


# tvert_lut = {  # TODO: implement this for the cases where the UV is not defined
# 	'down': [],
# 	'up': [],
# 	'north': [],
# 	'east': [],
# 	'south': [],
# 	'west': []
# }


def get_cube(
    down: str,
    up: str,
    north: str,
    east: str,
    south: str,
    west: str,
    transparency: BlockMeshTransparency = BlockMeshTransparency.FullOpaque,
    tint: tuple[int, int, int] = (1, 1, 1),
    bounds: BoundsType = ((0, 1), (0, 1), (0, 1)),
    texture_uv: TextureUVType = ((0, 0, 1, 1),) * 6,
    do_not_cull: tuple[bool, bool, bool, bool, bool, bool] = (
        False,
        False,
        False,
        False,
        False,
        False,
    ),
) -> BlockMesh:
    texture_paths: dict[str, int] = {}
    mesh_parts: list[tuple[list[Vertex], list[Triangle]] | None] = [
        None,
        None,
        None,
        None,
        None,
        None,
        None,
    ]
    box_coordinates = list(itertools.product(*bounds))
    tint_vec = FloatVec3(*tint)

    for (
        cull_direction,
        vertex_index,
        do_not_cull_face,
        texture_path,
        texture_coords,
    ) in zip(
        (
            BlockMeshCullDirection.CullDown,
            BlockMeshCullDirection.CullUp,
            BlockMeshCullDirection.CullNorth,
            BlockMeshCullDirection.CullEast,
            BlockMeshCullDirection.CullSouth,
            BlockMeshCullDirection.CullWest,
        ),
        VERTEX_INDEXES,
        do_not_cull,
        (down, up, north, east, south, west),
        texture_uv,
    ):
        # Get the index of the texture path. Add if it is not contained.
        texture_index = texture_paths.setdefault(texture_path, len(texture_paths))
        if do_not_cull_face:
            cull_direction = BlockMeshCullDirection.CullNone
        part = mesh_parts[cull_direction]
        if part is None:
            mesh_parts[cull_direction] = part = ([], [])
        verts, tris = part
        vert_count = len(verts)

        for i in range(4):
            x, y, z = box_coordinates[vertex_index[i]]
            uvx = texture_coords[UV_ROTATION_LUT[i * 2]]
            uvy = texture_coords[UV_ROTATION_LUT[i * 2 + 1]]
            verts.append(
                Vertex(
                    FloatVec3(x, y, z),
                    FloatVec2(uvx, uvy),
                    tint_vec,
                )
            )
        for a, b, c in TRI_FACE.reshape((2, 3)):
            tris.append(
                Triangle(a + vert_count, b + vert_count, c + vert_count, texture_index)
            )

    def create_part(
        part: tuple[list[Vertex], list[Triangle]] | None,
    ) -> BlockMeshPart | None:
        return None if part is None else BlockMeshPart(*part)

    return BlockMesh(
        transparency,
        list(texture_paths),
        (
            create_part(mesh_parts[0]),
            create_part(mesh_parts[1]),
            create_part(mesh_parts[2]),
            create_part(mesh_parts[3]),
            create_part(mesh_parts[4]),
            create_part(mesh_parts[5]),
            create_part(mesh_parts[6]),
        ),
    )


def get_unit_cube(
    down: str,
    up: str,
    north: str,
    east: str,
    south: str,
    west: str,
    transparency: BlockMeshTransparency = BlockMeshTransparency.FullOpaque,
    tint: tuple[int, int, int] = (1, 1, 1),
) -> BlockMesh:
    return get_cube(down, up, north, east, south, west, transparency, tint)



import os
import subprocess
import sys
from pathlib import Path
import platform
from tempfile import TemporaryDirectory

from setuptools import setup, Extension, Command
from setuptools.command.build_ext import build_ext

import versioneer

import requirements


def fix_path(path: str) -> str:
    return os.path.realpath(path).replace(os.sep, "/")


cmdclass: dict[str, type[Command]] = versioneer.get_cmdclass()


class CMakeBuild(cmdclass.get("build_ext", build_ext)):
    def build_extension(self, ext):
        import pybind11
        import amulet.pybind11_extensions
        import amulet.io
        import amulet.utils
        import amulet.nbt
        import amulet.core

        ext_dir = (
            (Path.cwd() / self.get_ext_fullpath("")).parent.resolve()
            / "amulet"
            / "resource_pack"
        )
        resource_pack_src_dir = (
            Path.cwd() / "src" / "amulet" / "resource_pack"
            if self.editable_mode
            else ext_dir
        )

        platform_args = []
        if sys.platform == "win32":
            platform_args.extend(["-G", "Visual Studio 17 2022"])
            if sys.maxsize > 2**32:
                platform_args.extend(["-A", "x64"])
            else:
                platform_args.extend(["-A", "Win32"])
            platform_args.extend(["-T", "v143"])
        elif sys.platform == "darwin":
            if platform.machine() == "arm64":
                platform_args.append("-DCMAKE_OSX_ARCHITECTURES=x86_64;arm64")

        if subprocess.run(["cmake", "--version"]).returncode:
            raise RuntimeError("Could not find cmake")
        with TemporaryDirectory() as tempdir:
            if subprocess.run(
                [
                    "cmake",
                    *platform_args,
                    f"-DPYTHON_EXECUTABLE={sys.executable}",
                    f"-Dpybind11_DIR={fix_path(pybind11.get_cmake_dir())}",
                    f"-Damulet_pybind11_extensions_DIR={fix_path(amulet.pybind11_extensions.__path__[0])}",
                    f"-Damulet_io_DIR={fix_path(amulet.io.__path__[0])}",
                    f"-Damulet_utils_DIR={fix_path(amulet.utils.__path__[0])}",
                    f"-Damulet_nbt_DIR={fix_path(amulet.nbt.__path__[0])}",
                    f"-Damulet_core_DIR={fix_path(amulet.core.__path__[0])}",
                    f"-Damulet_resource_pack_DIR={fix_path(resource_pack_src_dir)}",
                    f"-DAMULET_RESOURCE_PACK_EXT_DIR={fix_path(ext_dir)}",
                    f"-DCMAKE_INSTALL_PREFIX=install",
                    "-B",
                    tempdir,
                ]
            ).returncode:
                raise RuntimeError("Error configuring amulet-resource-pack")
            if subprocess.run(
                ["cmake", "--build", tempdir, "--config", "Release"]
            ).returncode:
                raise RuntimeError("Error building amulet-resource-pack")
            if subprocess.run(
                ["cmake", "--install", tempdir, "--config", "Release"]
            ).returncode:
                raise RuntimeError("Error installing amulet-resource-pack")


cmdclass["build_ext"] = CMakeBuild


setup(
    version=versioneer.get_version(),
    cmdclass=cmdclass,
    ext_modules=[Extension("amulet.resource_pack._amulet_resource_pack", [])]
    * (not os.environ.get("AMULET_SKIP_COMPILE", None)),
    install_requires=requirements.get_runtime_dependencies(),
)



from __future__ import annotations

import collections.abc
import typing

import numpy
from amulet.resource_pack.mesh.block._cube import get_cube, get_unit_cube

from . import _cube

__all__ = [
    "BlockMesh",
    "BlockMeshCullDirection",
    "BlockMeshPart",
    "BlockMeshTransparency",
    "CUBE_FACE_LUT",
    "FACE_KEYS",
    "FloatVec2",
    "FloatVec3",
    "TRI_FACE",
    "Triangle",
    "UV_ROTATION_LUT",
    "Vertex",
    "get_cube",
    "get_unit_cube",
    "merge_block_meshes",
]

class BlockMesh:
    """
    All the data that makes up a block mesh.
    """

    def __init__(
        self,
        transparency: BlockMeshTransparency,
        textures: collections.abc.Sequence[str],
        parts: tuple[
            amulet.resource_pack.mesh.block.BlockMeshPart | None,
            amulet.resource_pack.mesh.block.BlockMeshPart | None,
            amulet.resource_pack.mesh.block.BlockMeshPart | None,
            amulet.resource_pack.mesh.block.BlockMeshPart | None,
            amulet.resource_pack.mesh.block.BlockMeshPart | None,
            amulet.resource_pack.mesh.block.BlockMeshPart | None,
            amulet.resource_pack.mesh.block.BlockMeshPart | None,
        ],
    ) -> None: ...
    def rotate(self, rotx: typing.SupportsInt, roty: typing.SupportsInt) -> BlockMesh:
        """
        Rotate the mesh in the x and y axis. Accepted values are -3 to 3 which correspond to 90 degree rotations.
        """

    @property
    def parts(
        self,
    ) -> tuple[
        amulet.resource_pack.mesh.block.BlockMeshPart | None,
        amulet.resource_pack.mesh.block.BlockMeshPart | None,
        amulet.resource_pack.mesh.block.BlockMeshPart | None,
        amulet.resource_pack.mesh.block.BlockMeshPart | None,
        amulet.resource_pack.mesh.block.BlockMeshPart | None,
        amulet.resource_pack.mesh.block.BlockMeshPart | None,
        amulet.resource_pack.mesh.block.BlockMeshPart | None,
    ]:
        """
        The mesh parts that make up this mesh. The index corresponds to the value of BlockMeshCullDirection.
        """

    @property
    def textures(self) -> list[str]:
        """
        The texture paths used in this block mesh. The Triangle's texture_index attribute is an index into this list.
        """

    @property
    def transparency(self) -> BlockMeshTransparency:
        """
        The transparency state of this block mesh.
        """

class BlockMeshCullDirection:
    """
    The direction a mesh part is culled by. The value corresponds to the index in the mesh parts array.

    Members:

      CullNone : Is not culled by any neighbouring blocks.

      CullUp : Is culled by an opaque block above.

      CullDown : Is culled by an opaque block below.

      CullNorth : Is culled by an opaque block to the north.

      CullEast : Is culled by an opaque block to the east.

      CullSouth : Is culled by an opaque block to the south.

      CullWest : Is culled by an opaque block to the west.
    """

    CullDown: typing.ClassVar[
        BlockMeshCullDirection
    ]  # value = <BlockMeshCullDirection.CullDown: 2>
    CullEast: typing.ClassVar[
        BlockMeshCullDirection
    ]  # value = <BlockMeshCullDirection.CullEast: 4>
    CullNone: typing.ClassVar[
        BlockMeshCullDirection
    ]  # value = <BlockMeshCullDirection.CullNone: 0>
    CullNorth: typing.ClassVar[
        BlockMeshCullDirection
    ]  # value = <BlockMeshCullDirection.CullNorth: 3>
    CullSouth: typing.ClassVar[
        BlockMeshCullDirection
    ]  # value = <BlockMeshCullDirection.CullSouth: 5>
    CullUp: typing.ClassVar[
        BlockMeshCullDirection
    ]  # value = <BlockMeshCullDirection.CullUp: 1>
    CullWest: typing.ClassVar[
        BlockMeshCullDirection
    ]  # value = <BlockMeshCullDirection.CullWest: 6>
    __members__: typing.ClassVar[
        dict[str, BlockMeshCullDirection]
    ]  # value = {'CullNone': <BlockMeshCullDirection.CullNone: 0>, 'CullUp': <BlockMeshCullDirection.CullUp: 1>, 'CullDown': <BlockMeshCullDirection.CullDown: 2>, 'CullNorth': <BlockMeshCullDirection.CullNorth: 3>, 'CullEast': <BlockMeshCullDirection.CullEast: 4>, 'CullSouth': <BlockMeshCullDirection.CullSouth: 5>, 'CullWest': <BlockMeshCullDirection.CullWest: 6>}
    def __and__(self, other: typing.Any) -> typing.Any: ...
    def __eq__(self, other: typing.Any) -> bool: ...
    def __ge__(self, other: typing.Any) -> bool: ...
    def __gt__(self, other: typing.Any) -> bool: ...
    def __hash__(self) -> int: ...
    def __index__(self) -> int: ...
    def __init__(self, value: typing.SupportsInt) -> None: ...
    def __int__(self) -> int: ...
    def __invert__(self) -> typing.Any: ...
    def __le__(self, other: typing.Any) -> bool: ...
    def __lt__(self, other: typing.Any) -> bool: ...
    def __ne__(self, other: typing.Any) -> bool: ...
    def __or__(self, other: typing.Any) -> typing.Any: ...
    def __rand__(self, other: typing.Any) -> typing.Any: ...
    def __repr__(self) -> str: ...
    def __ror__(self, other: typing.Any) -> typing.Any: ...
    def __rxor__(self, other: typing.Any) -> typing.Any: ...
    def __str__(self) -> str: ...
    def __xor__(self, other: typing.Any) -> typing.Any: ...
    @property
    def name(self) -> str: ...
    @property
    def value(self) -> int: ...

class BlockMeshPart:
    """
    A part of a block mesh for one of the culling directions.
    """

    def __init__(
        self,
        verts: collections.abc.Sequence[Vertex],
        triangles: collections.abc.Sequence[Triangle],
    ) -> None: ...
    @property
    def triangles(self) -> list[Triangle]:
        """
        The triangles in this block mesh part.
        """

    @property
    def verts(self) -> list[Vertex]:
        """
        The vertices in this block mesh part.
        """

class BlockMeshTransparency:
    """
    The transparency of a block mesh.

    Members:

      FullOpaque : A block that occupies the whole block and is opaque.

      FullTranslucent : A block that occupies the whole block and has at least one translucent face.

      Partial : A block that does not occupy the whole block.
    """

    FullOpaque: typing.ClassVar[
        BlockMeshTransparency
    ]  # value = <BlockMeshTransparency.FullOpaque: 0>
    FullTranslucent: typing.ClassVar[
        BlockMeshTransparency
    ]  # value = <BlockMeshTransparency.FullTranslucent: 1>
    Partial: typing.ClassVar[
        BlockMeshTransparency
    ]  # value = <BlockMeshTransparency.Partial: 2>
    __members__: typing.ClassVar[
        dict[str, BlockMeshTransparency]
    ]  # value = {'FullOpaque': <BlockMeshTransparency.FullOpaque: 0>, 'FullTranslucent': <BlockMeshTransparency.FullTranslucent: 1>, 'Partial': <BlockMeshTransparency.Partial: 2>}
    def __eq__(self, other: typing.Any) -> bool: ...
    def __ge__(self, other: typing.Any) -> bool: ...
    def __gt__(self, other: typing.Any) -> bool: ...
    def __hash__(self) -> int: ...
    def __index__(self) -> int: ...
    def __init__(self, value: typing.SupportsInt) -> None: ...
    def __int__(self) -> int: ...
    def __le__(self, other: typing.Any) -> bool: ...
    def __lt__(self, other: typing.Any) -> bool: ...
    def __ne__(self, other: typing.Any) -> bool: ...
    def __repr__(self) -> str: ...
    def __str__(self) -> str: ...
    @property
    def name(self) -> str: ...
    @property
    def value(self) -> int: ...

class FloatVec2:
    """
    A 2D floating point vector
    """

    def __init__(self, x: typing.SupportsFloat, y: typing.SupportsFloat) -> None: ...
    @property
    def x(self) -> float: ...
    @property
    def y(self) -> float: ...

class FloatVec3:
    """
    A 3D floating point vector
    """

    def __init__(
        self, x: typing.SupportsFloat, y: typing.SupportsFloat, z: typing.SupportsFloat
    ) -> None: ...
    @property
    def x(self) -> float: ...
    @property
    def y(self) -> float: ...
    @property
    def z(self) -> float: ...

class Triangle:
    """
    The vertex and texture indexes that make up a triangle.
    """

    def __init__(
        self,
        vert_index_a: typing.SupportsInt,
        vert_index_b: typing.SupportsInt,
        vert_index_c: typing.SupportsInt,
        texture_index: typing.SupportsInt,
    ) -> None: ...
    @property
    def texture_index(self) -> int: ...
    @property
    def vert_index_a(self) -> int: ...
    @property
    def vert_index_b(self) -> int: ...
    @property
    def vert_index_c(self) -> int: ...

class Vertex:
    """
    Attributes for a single vertex.
    """

    def __init__(
        self, coord: FloatVec3, texture_coord: FloatVec2, tint: FloatVec3
    ) -> None: ...
    @property
    def coord(self) -> FloatVec3:
        """
        The spatial coordinate of the vertex.
        """

    @property
    def texture_coord(self) -> FloatVec2:
        """
        The texture coordinate of the vertex.
        """

    @property
    def tint(self) -> FloatVec3:
        """
        The tint colour for the vertex.
        """

def merge_block_meshes(meshes: collections.abc.Sequence[BlockMesh]) -> BlockMesh:
    """
    Merge multiple block mesh objects into one block mesh.
    """

CUBE_FACE_LUT: dict  # value = {'down': array([0, 4, 5, 1]), 'up': array([3, 7, 6, 2]), 'north': array([4, 0, 2, 6]), 'east': array([5, 4, 6, 7]), 'south': array([1, 5, 7, 3]), 'west': array([0, 1, 3, 2])}
FACE_KEYS: dict  # value = {None: <BlockMeshCullDirection.CullNone: 0>, 'up': <BlockMeshCullDirection.CullUp: 1>, 'down': <BlockMeshCullDirection.CullDown: 2>, 'north': <BlockMeshCullDirection.CullNorth: 3>, 'east': <BlockMeshCullDirection.CullEast: 4>, 'south': <BlockMeshCullDirection.CullSouth: 5>, 'west': <BlockMeshCullDirection.CullWest: 6>}
TRI_FACE: numpy.ndarray  # value = array([0, 1, 2, 0, 2, 3], dtype=uint32)
UV_ROTATION_LUT: list = [0, 3, 2, 3, 2, 1, 0, 1]



from typing import Optional, Iterator, TypeVar, Generic
import json

from amulet.core.block import Block, BlockStack
from amulet.resource_pack.mesh.block import (
    BlockMesh,
    merge_block_meshes,
    get_unit_cube,
)
from amulet.resource_pack.abc.resource_pack import BaseResourcePack
from amulet.utils.image import missing_no_icon_path
from amulet.utils.task_manager import AbstractProgressManager, VoidProgressManager

PackT = TypeVar("PackT", bound=BaseResourcePack)


class BaseResourcePackManager(Generic[PackT]):
    """The base class that all resource pack managers must inherit from. Defines the base api."""

    def __init__(self) -> None:
        self._packs: list[PackT] = []
        self._missing_block: Optional[BlockMesh] = None
        self._texture_is_transparent: dict[str, tuple[float, bool]] = {}
        self._cached_models: dict[BlockStack, BlockMesh] = {}

    @property
    def pack_paths(self) -> list[str]:
        return [pack.root_dir for pack in self._packs]

    def _unload(self) -> None:
        """Clear all loaded resources."""
        self._texture_is_transparent.clear()
        self._cached_models.clear()

    def _load_transparency_cache(self, path: str) -> None:
        try:
            with open(path) as f:
                self._texture_is_transparent = json.load(f)
        except:
            pass

    def _load(self, progress_manager: AbstractProgressManager) -> None:
        """Load resources."""
        raise NotImplementedError

    def reload(
        self, progress_manager: AbstractProgressManager = VoidProgressManager()
    ) -> None:
        """Unload and reload resources"""
        self._unload()
        self._load(progress_manager)

    @property
    def missing_no(self) -> str:
        """The path to the missing_no image"""
        return missing_no_icon_path

    @property
    def missing_block(self) -> BlockMesh:
        if self._missing_block is None:
            texture_path = self.get_texture_path("minecraft", "missing_no")
            self._missing_block = get_unit_cube(
                texture_path,
                texture_path,
                texture_path,
                texture_path,
                texture_path,
                texture_path,
            )
        return self._missing_block

    @property
    def textures(self) -> tuple[str, ...]:
        """Returns a tuple of all the texture paths in the resource pack."""
        raise NotImplementedError

    def get_texture_path(self, namespace: Optional[str], relative_path: str) -> str:
        """Get the absolute texture path from the namespace and relative path pair"""
        raise NotImplementedError

    def get_block_model(self, block_stack: BlockStack) -> BlockMesh:
        """Get a model for a block state.
        The block should already be in the resource pack format"""
        if block_stack not in self._cached_models:
            if len(block_stack) == 1:
                self._cached_models[block_stack] = self._get_model(
                    block_stack.base_block
                )
            else:
                self._cached_models[block_stack] = merge_block_meshes(
                    (self._get_model(block_stack.base_block),)
                    + tuple(
                        self._get_model(block_) for block_ in block_stack.extra_blocks
                    )
                )

        return self._cached_models[block_stack]

    def _get_model(self, block: Block) -> BlockMesh:
        raise NotImplementedError



import logging as _logging

from . import _version

__version__ = _version.get_versions()["version"]

# init a default logger
_logging.basicConfig(level=_logging.INFO, format="%(levelname)s - %(message)s")


def _init() -> None:
    import os
    import sys
    import ctypes

    if sys.platform == "win32":
        lib_path = os.path.join(os.path.dirname(__file__), "amulet_resource_pack.dll")
    elif sys.platform == "darwin":
        lib_path = os.path.join(
            os.path.dirname(__file__), "libamulet_resource_pack.dylib"
        )
    elif sys.platform == "linux":
        lib_path = os.path.join(os.path.dirname(__file__), "libamulet_resource_pack.so")
    else:
        raise RuntimeError(f"Unsupported platform {sys.platform}")

    # Import dependencies
    import amulet.utils
    import amulet.zlib
    import amulet.nbt
    import amulet.core

    # Load the shared library
    ctypes.cdll.LoadLibrary(lib_path)

    from ._amulet_resource_pack import init

    init(sys.modules[__name__])


_init()

from amulet.resource_pack.abc import (
    BaseResourcePack,
    BaseResourcePackManager,
)

from .unknown_resource_pack import UnknownResourcePack

from amulet.resource_pack.java import (
    JavaResourcePack,
    JavaResourcePackManager,
)

# from amulet.resource_pack.bedrock import (
#     BedrockResourcePack,
#     BedrockResourcePackManager,
# )

from ._load import load_resource_pack, load_resource_pack_manager
